<?php declare(strict_types=1);

namespace June\Core\Content\Flow\Dispatching\Action;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception;
use GuzzleHttp\Exception\GuzzleException;
use June\Components\Api\Client\JuneApiClient;
use June\Components\Logger\Helper\LoggerHelper;
use Shopware\Core\Content\ContactForm\Event\ContactFormEvent;
use Shopware\Core\Content\MailTemplate\Exception\MailEventConfigurationException;
use Shopware\Core\Content\MailTemplate\MailTemplateEntity;
use Shopware\Core\Content\MailTemplate\Subscriber\MailSendSubscriberConfig;
use Shopware\Core\Framework\Context;
use GuzzleHttp\Client;
use Shopware\Core\Content\Flow\Dispatching\Action\FlowAction;
use Shopware\Core\Framework\DataAbstractionLayer\Doctrine\FetchModeHelper;
use Shopware\Core\Framework\DataAbstractionLayer\EntityCollection;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\AndFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Event\MailAware;
use Shopware\Core\Framework\Event\OrderAware;
use Shopware\Core\Framework\Uuid\Uuid;
use June\Core\Framework\Event\JuneMailAware;
use Shopware\Core\Framework\Event\FlowEvent;
use Shopware\Core\System\SystemConfig\SystemConfigService;


class JuneMailAction extends FlowAction
{
    const JUNE_API_URL = 'June.config.juneapiurl';
    const JUNE_API_AUTH_TOKEN = 'June.config.juneauthtoken';
    const JUNE_TRANSFER_GUEST_DATA = 'June.config.transferFlowGuestData';

    /**
     * @var SystemConfigService
     */
    private SystemConfigService $configService;

    private EntityRepositoryInterface $juneFlowActionRepository;

    private LoggerHelper $loggerHelper;

    private JuneApiClient $juneApiClient;

    public function __construct
    (
        SystemConfigService $configService,
        EntityRepositoryInterface $juneFlowActionRepository,
        LoggerHelper $loggerHelper,
        JuneApiClient $juneApiClient
    )
    {
        $this->configService = $configService;
        $this->juneFlowActionRepository = $juneFlowActionRepository;
        $this->loggerHelper = $loggerHelper;
        $this->juneApiClient = $juneApiClient;
    }

    public static function getName(): string
    {
        // your own action name
        return 'action.june.mail';
    }

    public static function getSubscribedEvents(): array
    {
        return [
            self::getName() => 'handle',
        ];
    }

    public function requirements(): array
    {
        return [JuneMailAware::class];
    }

    public function handle(FlowEvent $event): void
    {
        $returnData = [];
        $config = $event->getConfig();
        $eventName = $event->getName();
        $userData = $event->getEvent()->getMailStruct()->getRecipients();
        $contentData = $this->getData($event->getEvent());
        $context = Context::createDefaultContext();
        $customerGuest = ($contentData['order'] ? $contentData['order']->getOrderCustomer()->getCustomer()->getGuest() : null);

        if(
            $this->configService->get(self::JUNE_TRANSFER_GUEST_DATA) == true
            || $customerGuest != true
        ) {
            try {
                if(strpos($eventName, "newsletter") !== false) {
                    foreach($userData as $email => $name) {
                        $returnData["email"] = $email;
                    }

                    $returnData["firstname"] = $contentData['newsletterRecipient']->getFirstName();
                    $returnData["lastname"] = $contentData['newsletterRecipient']->getLastName();

                    if($eventName === "newsletter.unsubscribe") {
                        $returnData["intent"] = "unsubscribe";
                    } else {
                        $returnData["intent"] = "subscribe";
                    }
                } else {
                    $returnData["content_data"] = "";
                    foreach($userData as $email => $name) {
                        $returnData["user_data"] = [];
                        $returnData["user_data"] = [
                            'email' => $email,
                            'name' => $name
                        ];
                    }

                    if($eventName == "checkout.order.placed") {
                        $returnData["content_data"] = [];
                        foreach($contentData as $keyData => $data) {
                            $returnData["content_data"][$keyData] = $data;
                            $returnData["content_data"][$keyData]->billing = [];
                            $billingId = $data->billingAddressId;
                            foreach ($data->addresses as $billingAddress) {
                                if($billingAddress->id == $billingId) {
                                    $returnData["content_data"][$keyData]->billing = [
                                        'firstname' => $billingAddress->firstName,
                                        'lastname' => $billingAddress->lastName,
                                        'street' => $billingAddress->street,
                                        'zipcode' => $billingAddress->zipcode,
                                        'city' => $billingAddress->city,
                                        "country" =>$billingAddress->country->name
                                    ];
                                }
                            }
                            foreach ($data->deliveries as $shippingAddress) {
                                $returnData["content_data"][$keyData]->shipping = [
                                    "firstname" => $shippingAddress->shippingOrderAddress->firstName,
                                    "lastname" => $shippingAddress->shippingOrderAddress->lastName,
                                    "street" => $shippingAddress->shippingOrderAddress->street,
                                    "zipcode" => $shippingAddress->shippingOrderAddress->zipcode,
                                    "city" => $shippingAddress->shippingOrderAddress->city,
                                    "country" =>$shippingAddress->shippingOrderAddress->country->name
                                ];
                            }
                        }
                    } else {
                        $returnData["content_data"] = [];
                        foreach($contentData as $keyData => $data) {
                            $returnData["content_data"][$keyData] = $data;
                        }
                    }
                    $returnData["email"] = $returnData["user_data"]["email"];
                    $returnData["action_type"] = $eventName;
                }

                $url = $this->configService->get(self::JUNE_API_URL);
                $authToken = $this->configService->get(self::JUNE_API_AUTH_TOKEN);

                if($config['juneFlowAction'] != NULL) {
                    $context = Context::createDefaultContext();
                    $criteria = new Criteria();

                    $criteria->addFilter(
                        new EqualsFilter('id', $config['juneFlowAction'] )
                    );
                    $result = $this->juneFlowActionRepository->search($criteria, $context)->first();

                    $collectToken = $result->getCollectToken();
                } else {
                    $this->loggerHelper->logError("There is no Collection configured in the FlowAction: {$eventName}", 'juneFlowActionConfig');
                }

                $this->juneApiClient->httpRequest($returnData, $url.$collectToken, $authToken);

            } catch (\Exception $e) {
                $this->loggerHelper->logError($e->getMessage(), 'juneFlowAction');
            }
        }
    }

    public function getData(MailAware $event): array
    {
        $data = [];

        foreach (array_keys($event::getAvailableData()->toArray()) as $key) {
            $getter = 'get' . ucfirst($key);
            if (!method_exists($event, $getter)) {
                throw new MailEventConfigurationException('Data for ' . $key . ' not available.', \get_class($event));
            }
            $data[$key] = $event->$getter();
        }

        return $data;
    }

    /*private function buildAttachments(MailAware $mailEvent, MailTemplateEntity $mailTemplate, MailSendSubscriberConfig $extensions, array $eventConfig): array
    {
        $attachments = [];

        if ($mailTemplate->getMedia() !== null) {
            foreach ($mailTemplate->getMedia() as $mailTemplateMedia) {
                if ($mailTemplateMedia->getMedia() === null) {
                    continue;
                }
                if ($mailTemplateMedia->getLanguageId() !== null && $mailTemplateMedia->getLanguageId() !== $mailEvent->getContext()->getLanguageId()) {
                    continue;
                }

                $attachments[] = $this->mediaService->getAttachment(
                    $mailTemplateMedia->getMedia(),
                    $mailEvent->getContext()
                );
            }
        }

        if (!empty($extensions->getMediaIds())) {
            $criteria = new Criteria($extensions->getMediaIds());
            $criteria->setTitle('send-mail::load-media');

            $entities = $this->mediaRepository->search($criteria, $mailEvent->getContext());

            foreach ($entities as $media) {
                $attachments[] = $this->mediaService->getAttachment($media, $mailEvent->getContext());
            }
        }

        if (!empty($extensions->getDocumentIds())) {
            $attachments = $this->buildOrderAttachments($extensions->getDocumentIds(), $attachments, $mailEvent->getContext());
        }

        if (empty($eventConfig['documentTypeIds']) || !\is_array($eventConfig['documentTypeIds']) || !$mailEvent instanceof OrderAware) {
            return $attachments;
        }

        return $this->buildFlowSettingAttachments($mailEvent->getOrderId(), $eventConfig['documentTypeIds'], $attachments, $mailEvent->getContext());
    }*/

    /*private function injectTranslator(MailAware $event): bool
    {
        if ($event->getSalesChannelId() === null) {
            return false;
        }

        if ($this->translator->getSnippetSetId() !== null) {
            return false;
        }

        $this->translator->injectSettings(
            $event->getSalesChannelId(),
            $event->getContext()->getLanguageId(),
            $this->languageLocaleProvider->getLocaleForLanguageId($event->getContext()->getLanguageId()),
            $event->getContext()
        );

        return true;
    }*/

    /*private function buildOrderAttachments(array $documentIds, array $attachments, Context $context): array
    {
        $criteria = new Criteria($documentIds);
        $criteria->setTitle('send-mail::load-attachments');
        $criteria->addAssociation('documentMediaFile');
        $criteria->addAssociation('documentType');

        $entities = $this->documentRepository->search($criteria, $context);

        return $this->mappingAttachmentsInfo($entities, $attachments, $context);
    }*/

    /*private function buildFlowSettingAttachments(string $orderId, array $documentTypeIds, array $attachments, Context $context): array
    {
        $documents = $this->connection->fetchAllAssociative(
            'SELECT
                LOWER(hex(`document`.`document_type_id`)) as doc_type,
                LOWER(hex(`document`.`id`)) as doc_id,
                `document`.`created_at` as newest_date
            FROM
                `document`
            WHERE
                HEX(`document`.`order_id`) = :orderId
                AND HEX(`document`.`document_type_id`) IN (:documentTypeIds)
            ORDER BY `document`.`created_at` DESC',
            [
                'orderId' => $orderId,
                'documentTypeIds' => $documentTypeIds,
            ],
            [
                'documentTypeIds' => Connection::PARAM_STR_ARRAY,
            ]
        );

        $documentsGroupByType = FetchModeHelper::group($documents);

        foreach ($documentsGroupByType as $document) {
            $documentIds[] = array_shift($document)['doc_id'];
        }

        if (empty($documentIds)) {
            return $attachments;
        }

        $criteria = new Criteria($documentIds);
        $criteria->setTitle('send-mail::load-flow-documents');
        $criteria->addAssociations(['documentMediaFile', 'documentType']);

        $entities = $this->documentRepository->search($criteria, $context);

        return $this->mappingAttachmentsInfo($entities, $attachments, $context);
    }*/

    /*private function mappingAttachmentsInfo(EntityCollection $entities, array $attachments, Context $context): array
    {
        foreach ($entities as $document) {
            $documentId = $document->getId();
            $document = $this->documentService->getDocument($document, $context);

            $attachments[] = [
                'id' => $documentId,
                'content' => $document->getFileBlob(),
                'fileName' => $document->getFilename(),
                'mimeType' => $document->getContentType(),
            ];
        }

        return $attachments;
    }*/
}
