<?php

namespace June\Service\ScheduledTask;

use GuzzleHttp\Exception\GuzzleException;
use June\Components\Api\Client\JuneApiClient;
use June\Components\Logger\Helper\LoggerHelper;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\RangeFilter;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskDefinition;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskEntity;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskHandler;
use Shopware\Core\System\SalesChannel\Context\AbstractSalesChannelContextFactory;
use Shopware\Core\System\SalesChannel\Entity\SalesChannelRepositoryInterface;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ExportOrdersTaskHandler extends ScheduledTaskHandler {

    const JUNE_COMMERCE_TOKEN = "June.config.commercetoken";

    private EntityRepositoryInterface $orderRepository;

    private EntityRepositoryInterface $pluginRepository;

    private EntityRepositoryInterface $orderLineItemsRepository;

    private SalesChannelRepositoryInterface $categoryRepository;

    private AbstractSalesChannelContextFactory $salesChannelContextFactory;

    private EntityRepositoryInterface $salesRepository;

    private SystemConfigService $configService;

    private LoggerHelper $loggerHelper;

    private JuneApiClient $juneApiClient;

    public function __construct
    (
        EntityRepositoryInterface $scheduledTaskRepository,
        EntityRepositoryInterface $pluginRepository,
        EntityRepositoryInterface $orderRepository,
        EntityRepositoryInterface $orderLineItemsRepository,
        SalesChannelRepositoryInterface $categoryRepository,
        AbstractSalesChannelContextFactory $salesChannelContextFactory,
        EntityRepositoryInterface $salesRepository,
        SystemConfigService $configService,
        LoggerHelper $loggerHelper,
        JuneApiClient $juneApiClient
    )
    {
        parent::__construct($scheduledTaskRepository);
        $this->orderRepository = $orderRepository;
        $this->pluginRepository = $pluginRepository;
        $this->orderLineItemsRepository = $orderLineItemsRepository;
        $this->categoryRepository = $categoryRepository;
        $this->salesChannelContextFactory = $salesChannelContextFactory;
        $this->salesRepository = $salesRepository;
        $this->configService = $configService;
        $this->loggerHelper = $loggerHelper;
        $this->juneApiClient = $juneApiClient;
    }

    public static function getHandledMessages(): iterable
    {

        return [ ExportOrdersTask::class ];
    }

    public function run(): void
    {
        if($this->configService->get('June.config.startExport') === true) {
            $data = [];
            $orders = $this->getOrders();

            foreach($orders as $order) {
                $voucher = 0;
                $products = [];
                $itemCriteria = new Criteria();
                $itemCriteria->addFilter(new EqualsFilter("orderId", $order->getId()));
                $shopUrl = $this->getShopUrl($order->getSalesChannelId(), $order->getLanguageId());
                $lineItems = $this->orderLineItemsRepository->search($itemCriteria, Context::createDefaultContext())->getElements();

                foreach ($lineItems as $item) {
                    if($item->getType() === "promotion") {
                        $voucher = $voucher + abs($item->getPrice()->getTotalPrice());
                    } else {
                        $categoryIds = [];
                        $categories = [];
                        $payload = (array)$item->getPayload();
                        foreach ($payload as $key => $value) {
                            if($key === 'categoryIds') {
                                $categoryIds = $value;
                            }
                        }
                        foreach($categoryIds as $categoryId) {
                            $categoryCriteria = new Criteria();
                            $categoryCriteria = $categoryCriteria->addFilter(new EqualsFilter('id', $categoryId));
                            $category = $this->categoryRepository->search($categoryCriteria, $this->createSalesChannelContext($order->getSalesChannelId()))->first();
                            $category = $category->getName();
                            array_push($categories, $category);
                        }
                    }

                    $product = [
                        'id' => $item->getId(),
                        'name' => $item->getLabel(),
                        'link' => $shopUrl."/detail/".$item->getProductId(),
                        'price' => $item->getPrice()->getTotalPrice(),
                        'categories' => $categories,
                        'qty' => $item->getQuantity()
                    ];
                    if($shopUrl != null){
                        $product['link'] = $shopUrl."/detail/".$item->getProductId();
                    }
                    array_push($products, $product);
                }

                $enriched_order = [
                    'id' => $order->getOrderCustomer()->getId(),
                    'email' => $order->getOrderCustomer()->getEmail(),
                    'customer' => [
                        'firstname' => $order->getOrderCustomer()->getFirstname(),
                        'lastname' => $order->getOrderCustomer()->getLastname(),
                        'store_id' => $order->getSalesChannelId()
                    ],
                    'order' => [
                        'id' => $order->getId(),
                        'created_at' => $order->getOrderDateTime()->format('c'),
                        'total' => $order->getPrice()->getTotalPrice(),
                        'shipping' => $order->getShippingCosts()->getTotalPrice(),
                        'voucher' => $voucher,
                        'products' => $products
                    ]
                ];
                array_push($data, $enriched_order);
            }

            $this->juneApiClient->sendData($data, "https://engagement.juneapp.com/commerce/webhook/".$this->configService->get(self::JUNE_COMMERCE_TOKEN)."?import=true", null);

            $offset = $this->configService->get('June.config.orderOffset') + 20;
            $this->configService->set('June.config.orderOffset', $offset);
        } else {
            return;
        }
    }

    public function handle($task): void
    {
        $taskId = $task->getTaskId();

        if ($taskId === null) {
            // run task independent of the schedule
            $this->run();

            return;
        }

        /** @var ScheduledTaskEntity|null $taskEntity */
        $taskEntity = $this->scheduledTaskRepository
            ->search(new Criteria([$taskId]), Context::createDefaultContext())
            ->get($taskId);

        if ($taskEntity === null || !$taskEntity->isExecutionAllowed()) {
            return;
        }

        $this->markTaskRunning($task);

        try {
            $this->run();
        } catch (\Throwable $e) {
            $this->markTaskFailed($task);

            $this->loggerHelper->logError($e->getMessage(), "orderExport");
        }

        if($this->configService->get('June.config.startExport') === true) {
            $orders = $this->getOrders();
            if (count($orders) > 0) {
                $this->rescheduleTask($task, $taskEntity);
            } else {
                $this->scheduledTaskRepository->update([
                    [
                        'id' => $task->getTaskId(),
                        'status' => ScheduledTaskDefinition::STATUS_INACTIVE,
                    ],
                ], Context::createDefaultContext());
            }
        } else {
            $this->rescheduleTask($task, $taskEntity);
        }
    }

    public function createSalesChannelContext(string $salesChannelId): SalesChannelContext
    {
        return $this->salesChannelContextFactory->create('', $salesChannelId, []);
    }

    private function getShopUrl($salesChannelId, $languageId) {
        $salesChannelUrls = [];
        $salesCriteria = new Criteria();
        $salesCriteria = $salesCriteria->addAssociation('domains');
        $salesChannelIds = $this->salesRepository->search($salesCriteria, Context::createDefaultContext());
        foreach($salesChannelIds->getEntities()->getElements() as $key => $salesChannel){
            foreach($salesChannel->getDomains() as $element) {
                array_push($salesChannelUrls, $element);
            }
        }
        foreach ($salesChannelUrls as $url) {
            if($url->getSalesChannelId() === $salesChannelId && $url->getLanguageId() === $languageId) {
                return $url->getUrl();
            }
        }
        return null;
    }

    private function getOrders() {
        $limit = 20;
        $pluginCriteria = new Criteria();
        $pluginCriteria = $pluginCriteria->addFilter(new EqualsFilter('name', 'June'));
        $plugin = $this->pluginRepository->search($pluginCriteria, Context::createDefaultContext())->first();
        $installDate = $plugin->getInstalledAt()->format('c');

        if($this->configService->get('June.config.orderOffset')) {
            $offset = $this->configService->get('June.config.orderOffset');
        } else {
            $offset = 0;
        }

        $orderCriteria = new Criteria();
        $orderCriteria->addFilter(new RangeFilter('orderDateTime',
            [
                RangeFilter::LTE => $installDate
            ]
        ));
        $orderCriteria->setLimit($limit);
        $orderCriteria->setOffset($offset);

        return $orders = $this->orderRepository->search($orderCriteria, Context::createDefaultContext())->getElements();
    }
}
