<?php

namespace June\Subscriber;

use June\Components\Api\Client\JuneApiClient;
use Shopware\Core\Checkout\Cart\Event\CartChangedEvent;
use Shopware\Core\Checkout\Customer\Event\GuestCustomerRegisterEvent;
use Shopware\Core\Checkout\Order\Event\OrderStateMachineStateChangeEvent;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Order\OrderEvents;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;
use Shopware\Core\System\SalesChannel\Context\AbstractSalesChannelContextFactory;
use Shopware\Core\System\SalesChannel\Entity\SalesChannelRepositoryInterface;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Shopware\Core\Checkout\Cart\Event\CheckoutOrderPlacedEvent;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Context;
use June\Components\Logger\Helper\LoggerHelper;

class JuneCommerceSubscriber implements EventSubscriberInterface
{
    const JUNE_COMMERCE_TOKEN = "June.config.commercetoken";
    const JUNE_TRANSFER_ORDER = "June.config.orderCommerceData";
    const JUNE_TRANSFER_FULFILLMENT = "June.config.fulfillCommerceData";
    const JUNE_TRANSFER_CART = "June.config.cartCommerceData";
    const JUNE_TRANSFER_GUESTDATA = "June.config.transferGuestData";

    private EntityRepositoryInterface $salesRepository;

    private EntityRepositoryInterface $orderRepository;

    private EntityRepositoryInterface $orderLineItemsRepository;

    private EntityRepositoryInterface $languageRepository;

    private EntityRepositoryInterface $localeRepository;

    private EntityRepositoryInterface $countryRepository;

    private SalesChannelRepositoryInterface $categoryRepository;

    private AbstractSalesChannelContextFactory $salesChannelContextFactory;

    private SystemConfigService $configService;

    private LoggerHelper $loggerHelper;

    private JuneApiClient $juneApiClient;

    public function __construct
    (
        EntityRepositoryInterface $salesRepository,
        EntityRepositoryInterface $orderRepository,
        EntityRepositoryInterface $orderLineItemsRepository,
        EntityRepositoryInterface $languageRepository,
        EntityRepositoryInterface $localeRepository,
        EntityRepositoryInterface $countryRepository,
        SalesChannelRepositoryInterface $categoryRepository,
        AbstractSalesChannelContextFactory $salesChannelContextFactory,
        SystemConfigService $configService,
        LoggerHelper $loggerHelper,
        JuneApiClient $juneApiClient
    )
    {
        $this->salesRepository = $salesRepository;
        $this->orderRepository = $orderRepository;
        $this->orderLineItemsRepository = $orderLineItemsRepository;
        $this->languageRepository = $languageRepository;
        $this->localeRepository = $localeRepository;
        $this->countryRepository = $countryRepository;
        $this->categoryRepository = $categoryRepository;
        $this->salesChannelContextFactory = $salesChannelContextFactory;
        $this->configService = $configService;
        $this->loggerHelper = $loggerHelper;
        $this->juneApiClient = $juneApiClient;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutOrderPlacedEvent::class => 'orderPlaced',
            CartChangedEvent::class => 'cartChanged',
            OrderEvents::ORDER_WRITTEN_EVENT => 'orderFulfilled',
            GuestCustomerRegisterEvent::class => 'guestRegister',
        ];
    }

    public function orderPlaced(CheckoutOrderPlacedEvent $event) {
        if($this->configService->get(self::JUNE_COMMERCE_TOKEN) != null || $this->configService->get(self::JUNE_COMMERCE_TOKEN) === "") {
            if($this->configService->get(self::JUNE_TRANSFER_ORDER)) {
                if($this->configService->get(self::JUNE_TRANSFER_GUESTDATA) === false && $event->getOrder()->getOrderCustomer()->getCustomer()->getGuest() === false || $this->configService->get(self::JUNE_TRANSFER_GUESTDATA) === true || is_null($this->configService->get(self::JUNE_TRANSFER_GUESTDATA))) {
                    try {
                        $voucher = 0;
                        $shopUrl = $this->getShopUrl($event->getOrder()->getSalesChannelId(), $event->getOrder()->getOrderCustomer()->getCustomer()->getLanguageId());

                        $recipient = array_keys($event->getMailStruct()->getRecipients());
                        $orderDate = $event->getOrder()->getOrderDateTime()->format('c');
                        $products = [];

                        $city = "";
                        $billingId = $event->getOrder()->getBillingAddressId();
                        foreach ($event->getOrder()->getAddresses() as $address) {
                            if($address->getId() == $billingId) {
                                $city = $address->getCity();
                            }
                        }

                        foreach ($event->getOrder()->getLineItems() as $item) {
                            if($item->getType() === "promotion") {
                                $voucher = $voucher + abs($item->getTotalPrice());
                            } else {
                                $categories = [];
                                foreach($item->getPayload()['categoryIds'] as $categoryId) {
                                    $categoryCriteria = new Criteria();
                                    $categoryCriteria = $categoryCriteria->addFilter(new EqualsFilter('id', $categoryId));
                                    $category = $this->categoryRepository->search($categoryCriteria, $this->createSalesChannelContext($event->getSalesChannelId()))->first();
                                    $category = $category->getName();
                                    array_push($categories, $category);
                                }

                                $product = [
                                    'id' => $item->getId(),
                                    'name' => $item->getLabel(),
                                    'qty' => $item->getQuantity(),
                                    'price' => $item->getTotalPrice(),
                                    'category' => $categories
                                ];

                                if($item->getCover() != NULL) {
                                    $product['image'] = $item->getCover()->getUrl();
                                }

                                if($shopUrl != null){
                                    $product['link'] = $shopUrl."/detail/".$item->getProductId();
                                }
                                array_push($products, $product);
                            }
                        }

                        $data = [
                            'id' => $event->getOrder()->getOrderCustomer()->getId(),
                            'email' => $recipient[0],
                            'customer' => [
                                'firstname' => $event->getOrder()->getOrderCustomer()->getFirstname(),
                                'lastname' => $event->getOrder()->getOrderCustomer()->getLastname(),
                                'store_id' => $event->getOrder()->getSalesChannelId(),
                                'locale' => $this->getLanguageCode($event->getOrder()->getOrderCustomer()->getCustomer()->getLanguageId()),
                                'geo_location' => [
                                    'city' => $city,
                                    'country' => $this->getCountry($event->getOrder()->getSalesChannelId()),
                                ],
                                'guest' => $event->getOrder()->getOrderCustomer()->getCustomer()->getGuest()
                            ],
                            'order' => [
                                'id' => $event->getOrder()->getId(),
                                'created_at' => $orderDate,
                                'fulfilled_at' => null,
                                'total' => $event->getOrder()->getPrice()->getTotalPrice(),
                                'shipping' => $event->getOrder()->getShippingCosts()->getTotalPrice(),
                                'voucher' => $voucher,
                                'products' => $products,
                            ],
                        ];

                        $this->juneApiClient->sendData($data,"https://engagement.juneapp.com/commerce/webhook/".$this->configService->get(self::JUNE_COMMERCE_TOKEN), null);

                    } catch(\Exception $e) {
                        $this->loggerHelper->logError($e->getMessage(), "commerceOrderPlaced");
                    }
                }
            }
        } else {
            $this->loggerHelper->logError("The Commerce Token in the JUNE Plugin configuration is missing", "pluginConfig");
        }
    }

    public function cartChanged(CartChangedEvent $event) {
        if($this->configService->get(self::JUNE_COMMERCE_TOKEN) != null || $this->configService->get(self::JUNE_COMMERCE_TOKEN) === "") {
            if($this->configService->get(self::JUNE_TRANSFER_CART)) {
                if($this->configService->get(self::JUNE_TRANSFER_GUESTDATA) === false && $event->getContext()->getCustomer()->getGuest() === false || $this->configService->get(self::JUNE_TRANSFER_GUESTDATA) === true || is_null($this->configService->get(self::JUNE_TRANSFER_GUESTDATA))) {
                    try {
                        $customer = $event->getContext()->getCustomer();
                        $shopUrl = $this->getShopUrl($event->getContext()->getSalesChannelId(), $event->getContext()->getLanguageId());
                        $products = [];
                        $data = [];

                        if ($customer != null) {
                            $data['id'] = $customer->getId();
                            $data['email'] = $customer->getEmail();
                            $data['customer'] = [
                                'firstname' => $event->getContext()->getCustomer()->getFirstName(),
                                'lastname' => $event->getContext()->getCustomer()->getLastName(),
                                'store_id' => $event->getContext()->getCustomer()->getSalesChannelId(),
                                'locale' => $this->getLanguageCode($event->getContext()->getCustomer()->getLanguageId()),
                                'geo_location' => [
                                    'city' => $event->getContext()->getCustomer()->getDefaultBillingAddress()->getCity(),
                                    'continent' => '',
                                    'country' => $this->getCountry($event->getContext()->getSalesChannelId()),
                                ],
                                'guest' => $event->getContext()->getCustomer()->getGuest(),
                            ];
                        } else {
                            $data['id'] = session_id();
                        }

                        foreach ($event->getCart()->getLineItems() as $item) {
                            if ($item->getType() !== "promotion") {
                                $categories = [];
                                foreach ($item->getPayload()['categoryIds'] as $categoryId) {
                                    $categoryCriteria = new Criteria();
                                    $categoryCriteria = $categoryCriteria->addFilter(new EqualsFilter('id', $categoryId));
                                    $category = $this->categoryRepository->search($categoryCriteria, $this->createSalesChannelContext($event->getContext()->getSalesChannelId()))->first();
                                    $category = $category->getName();
                                    array_push($categories, $category);
                                }
                                $product = [
                                    'id' => $item->getId(),
                                    'name' => $item->getLabel(),
                                    'link' => $shopUrl . '/detail/' . $item->getId(),
                                    'qty' => $item->getQuantity(),
                                    'price' => $item->getPrice()->getTotalPrice(),
                                    'category' => $categories
                                ];

                                if ($item->getCover() != NULL) {
                                    $product['image'] = $item->getCover()->getUrl();
                                }

                                array_push($products, $product);
                            }
                        }

                        $data['cart'] = [
                            'action' => 'update',
                            'products' => $products
                        ];

                        $this->juneApiClient->sendData($data, "https://engagement.juneapp.com/commerce/webhook/" . $this->configService->get(self::JUNE_COMMERCE_TOKEN), null);
                    } catch (\Exception $e) {
                        $this->loggerHelper->logError($e->getMessage(), "commerceCartChanged");
                    }
                }
            }
        } else {
            $this->loggerHelper->logError("The Commerce Token in the JUNE Plugin configuration is missing", "pluginConfig");
        }
    }

    public function orderFulfilled(EntityWrittenEvent $event): void {
        if($this->configService->get(self::JUNE_COMMERCE_TOKEN) != null || $this->configService->get(self::JUNE_COMMERCE_TOKEN) === "") {
            if ($this->configService->get(self::JUNE_TRANSFER_FULFILLMENT)) {
                $event_data = $event->getPayloads();
                if (isset($event_data[0]["id"])) {
                    $order = $this->getOrder($event_data[0]["id"], Context::createDefaultContext());
                    $products = [];

                    $itemCriteria = new Criteria();
                    $itemCriteria->addFilter(new EqualsFilter("orderId", $order->getId()));
                    $lineItems = $this->orderLineItemsRepository->search($itemCriteria, Context::createDefaultContext())->getElements();


                    if ($order->getStateMachineState()->getTechnicalName() === "completed") {
                        $voucher = 0;
                        $shopUrl = $this->getShopUrl($order->getSalesChannelId(), $order->getLanguageId());
                        $orderDate = $order->getOrderDateTime()->format('c');

                        $city = "";

                        foreach ($lineItems as $item) {
                            if ($item->getType() === "promotion") {
                                $voucher = $voucher + abs($item->getTotalPrice());
                            } else {
                                $categories = [];
                                foreach ($item->getPayload()['categoryIds'] as $categoryId) {
                                    $categoryCriteria = new Criteria();
                                    $categoryCriteria = $categoryCriteria->addFilter(new EqualsFilter('id', $categoryId));
                                    $category = $this->categoryRepository->search($categoryCriteria, $this->createSalesChannelContext($order->getSalesChannelId()))->first();
                                    $category = $category->getName();
                                    array_push($categories, $category);
                                }

                                $product = [
                                    'id' => $item->getId(),
                                    'name' => $item->getLabel(),
                                    'qty' => $item->getQuantity(),
                                    'price' => $item->getTotalPrice(),
                                    'category' => $categories
                                ];

                                if ($item->getCover() != NULL) {
                                    $product['image'] = $item->getCover()->getUrl();
                                }

                                if ($shopUrl != null) {
                                    $product['link'] = $shopUrl . "/detail/" . $item->getProductId();
                                }
                                array_push($products, $product);
                            }
                        }

                        $data = [
                            'id' => $order->getOrderCustomer()->getId(),
                            'email' => $order->getOrderCustomer()->getEmail(),
                            'customer' => [
                                'firstname' => $order->getOrderCustomer()->getFirstname(),
                                'lastname' => $order->getOrderCustomer()->getLastname(),
                                'store_id' => $order->getSalesChannelId(),
                                'locale' => $this->getLanguageCode($order->getLanguageId()),
                                'geo_location' => [
                                    'city' => $city,
                                    'country' => $this->getCountry($order->getSalesChannelId()),
                                ],
                                'guest' => null
                            ],
                            'fulfillment' => [
                                'id' => $order->getId(),
                                'created_at' => $orderDate,
                                'fulfilled_at' => null,
                                'total' => $order->getPrice()->getTotalPrice(),
                                'shipping' => $order->getShippingCosts()->getTotalPrice(),
                                'voucher' => $voucher,
                                'products' => $products,
                            ],
                        ];

                        $this->juneApiClient->sendData($data, "https://engagement.juneapp.com/commerce/webhook/" . $this->configService->get(self::JUNE_COMMERCE_TOKEN), null);
                    }
                }
            }
        }
    }

    public function guestRegister(GuestCustomerRegisterEvent $event) {
        if($this->configService->get(self::JUNE_TRANSFER_CART)) {
            if($this->configService->get(self::JUNE_TRANSFER_GUESTDATA)) {

                $customer = $event->getCustomer();
                $data = [
                    "id" => session_id(),
                    "email" => $customer->getEmail(),
                    "customer" => [
                        "firstname" => $customer->getFirstName(),
                        "lastname" => $customer->getLastName(),
                        "store_id" => $event->getSalesChannelId(),
                        "locale" => $this->getLanguageCode($event->getSalesChannelContext()->getLanguageId()),
                        "geo_location" => [
                            "city" => $customer->getDefaultBillingAddress()->getCity(),
                            "continent" => "",
                            "country" => $customer->getDefaultBillingAddress()->getCountry()->getName()
                        ],
                        "guest" => true
                    ]
                ];

                $this->juneApiClient->sendData($data,"https://engagement.juneapp.com/commerce/webhook/".$this->configService->get(self::JUNE_COMMERCE_TOKEN), null);
            }
        }
    }

    public function createSalesChannelContext(string $salesChannelId): SalesChannelContext
    {
        return $this->salesChannelContextFactory->create('', $salesChannelId, []);
    }

    private function createUserId(): string {
        $client_ip = $this->getClientIp();
        $user_agent = $_SERVER['HTTP_USER_AGENT'];

        return md5($client_ip.$user_agent);
    }

    private function getClientIp() {
        if (array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)){
            return  $_SERVER["HTTP_X_FORWARDED_FOR"];
        }else if (array_key_exists('REMOTE_ADDR', $_SERVER)) {
            return $_SERVER["REMOTE_ADDR"];
        }else if (array_key_exists('HTTP_CLIENT_IP', $_SERVER)) {
            return $_SERVER["HTTP_CLIENT_IP"];
        }
    }

    private function getShopUrl($salesChannelId, $languageId) {
        $salesChannelUrls = [];
        $salesCriteria = new Criteria();
        $salesCriteria = $salesCriteria->addAssociation('domains');
        $salesChannelIds = $this->salesRepository->search($salesCriteria, Context::createDefaultContext());
        foreach($salesChannelIds->getEntities()->getElements() as $key => $salesChannel){
            foreach($salesChannel->getDomains() as $element) {
                array_push($salesChannelUrls, $element);
            }
        }
        foreach ($salesChannelUrls as $url) {
            if($url->getSalesChannelId() === $salesChannelId && $url->getLanguageId() === $languageId) {
                return $url->getUrl();
            }
        }
        return null;
    }

    private function getLanguageCode($languageId) {
        $languageCriteria = new Criteria();
        $languageCriteria = $languageCriteria->addFilter(new EqualsFilter('id', $languageId));
        $language = $this->languageRepository->search($languageCriteria, Context::createDefaultContext())->first();

        $localeCriteria = new Criteria();
        $localeCriteria = $localeCriteria->addFilter(new EqualsFilter('id', $language->getLocaleId()));
        $locale = $this->localeRepository->search($localeCriteria, Context::createDefaultContext())->first();

        return $locale->getCode();
    }

    private function getCountry($salesChannelId) {
        $salesChannelCriteria = new Criteria();
        $salesChannelCriteria = $salesChannelCriteria->addFilter(new EqualsFilter('id', $salesChannelId));
        $salesChannel = $this->salesRepository->search($salesChannelCriteria, Context::createDefaultContext())->first();

        $countryCriteria = new Criteria();
        $countryCriteria = $countryCriteria->addFilter(new EqualsFilter('id', $salesChannel->getCountryId()));
        $country = $this->countryRepository->search($countryCriteria, Context::createDefaultContext())->first();

        return $country->getName();
    }

    private function getOrder(string $orderId, Context $context): OrderEntity
    {
        $orderCriteria = new Criteria();
        $orderCriteria = $orderCriteria->addFilter(new EqualsFilter('id', $orderId));

        $order = $this->orderRepository
            ->search($orderCriteria, $context)
            ->first();

        return $order;
    }

}
